summaryrefslogtreecommitdiff
path: root/src/rebar_config.erl
diff options
context:
space:
mode:
authorStuart Thackray <stuart.thackray@gmail.com>2017-01-06 12:57:03 +0200
committerStuart Thackray <stuart.thackray@gmail.com>2017-01-06 12:57:03 +0200
commit71187514dabdd94aa333495d92df84a2e750099f (patch)
tree74e3257522e3c9697a8b3aa1fa3a589d8a3ddf9b /src/rebar_config.erl
parent8fb8a91130e0e5c239bb837b9adc4ad71e5806b8 (diff)
parent91a25c11841314e35b095ea0382867c090616ff1 (diff)
Merged branch master into master
Diffstat (limited to 'src/rebar_config.erl')
-rw-r--r--src/rebar_config.erl94
1 files changed, 82 insertions, 12 deletions
diff --git a/src/rebar_config.erl b/src/rebar_config.erl
index 836980e..b50c030 100644
--- a/src/rebar_config.erl
+++ b/src/rebar_config.erl
@@ -56,35 +56,105 @@ consult_lock_file(File) ->
[] ->
[];
[Locks] when is_list(Locks) -> % beta lock file
- Locks;
+ read_attrs(beta, Locks, []);
[{Vsn, Locks}|Attrs] when is_list(Locks) -> % versioned lock file
- %% Make sure the warning above is to be shown whenever a version
- %% newer than the current one is being used, as we can't parse
- %% all the contents of the lock file properly.
+ %% Because this is the first version of rebar3 to introduce a lock
+ %% file, all versionned lock files with a different versions have
+ %% to be newer.
+ case Vsn of
+ ?CONFIG_VERSION ->
+ ok;
+ _ ->
+ %% Make sure the warning below is to be shown whenever a version
+ %% newer than the current one is being used, as we can't parse
+ %% all the contents of the lock file properly.
+ warn_vsn_once()
+ end,
+ read_attrs(Vsn, Locks, Attrs)
+ end.
+
+warn_vsn_once() ->
+ Warn = application:get_env(rebar, warn_config_vsn) =/= {ok, false},
+ application:set_env(rebar, warn_config_vsn, false),
+ case Warn of
+ false -> ok;
+ true ->
?WARN("Rebar3 detected a lock file from a newer version. "
"It will be loaded in compatibility mode, but important "
"information may be missing or lost. It is recommended to "
- "upgrade Rebar3.", []),
- read_attrs(Vsn, Locks, Attrs)
+ "upgrade Rebar3.", [])
end.
+
write_lock_file(LockFile, Locks) ->
- NewLocks = write_attrs(Locks),
+ {NewLocks, Attrs} = write_attrs(Locks),
%% Write locks in the beta format, at least until it's been long
%% enough we can start modifying the lock format.
- file:write_file(LockFile, io_lib:format("~p.~n", [NewLocks])).
+ case Attrs of
+ [] -> % write the old beta copy to avoid changes
+ file:write_file(LockFile, io_lib:format("~p.~n", [NewLocks]));
+ _ ->
+ file:write_file(LockFile,
+ io_lib:format("{~p,~n~p}.~n[~n~s~n].~n",
+ [?CONFIG_VERSION, NewLocks,
+ format_attrs(Attrs)]))
+ end.
-read_attrs(_Vsn, Locks, _Attrs) ->
+%% Attributes have a special formatting to ensure there's only one per
+%% line in terms of pkg_hash, so we disturb source diffing as little
+%% as possible.
+format_attrs([]) -> [];
+format_attrs([{pkg_hash, Vals}|T]) ->
+ [io_lib:format("{pkg_hash,[~n",[]), format_hashes(Vals), "]}",
+ maybe_comma(T) | format_attrs(T)];
+format_attrs([H|T]) ->
+ [io_lib:format("~p~s", [H, maybe_comma(T)]) | format_attrs(T)].
+
+format_hashes([]) -> [];
+format_hashes([{Pkg,Hash}|T]) ->
+ [" {", io_lib:format("~p",[Pkg]), ", ", io_lib:format("~p", [Hash]), "}",
+ maybe_comma(T) | format_hashes(T)].
+
+maybe_comma([]) -> "";
+maybe_comma([_|_]) -> io_lib:format(",~n", []).
+
+read_attrs(_Vsn, Locks, Attrs) ->
%% Beta copy does not know how to expand attributes, but
%% is ready to support it.
- Locks.
+ expand_locks(Locks, extract_pkg_hashes(Attrs)).
+
+extract_pkg_hashes(Attrs) ->
+ Props = case Attrs of
+ [First|_] -> First;
+ [] -> []
+ end,
+ proplists:get_value(pkg_hash, Props, []).
+
+expand_locks([], _Hashes) ->
+ [];
+expand_locks([{Name, {pkg,PkgName,Vsn}, Lvl} | Locks], Hashes) ->
+ Hash = proplists:get_value(Name, Hashes),
+ [{Name, {pkg,PkgName,Vsn,Hash}, Lvl} | expand_locks(Locks, Hashes)];
+expand_locks([Lock|Locks], Hashes) ->
+ [Lock | expand_locks(Locks, Hashes)].
write_attrs(Locks) ->
%% No attribute known that needs to be taken out of the structure,
%% just return terms as is.
- Locks.
-
+ {NewLocks, Hashes} = split_locks(Locks, [], []),
+ case Hashes of
+ [] -> {NewLocks, []};
+ _ -> {NewLocks, [{pkg_hash, lists:sort(Hashes)}]}
+ end.
+split_locks([], Locks, Hashes) ->
+ {lists:reverse(Locks), Hashes};
+split_locks([{Name, {pkg,PkgName,Vsn,undefined}, Lvl} | Locks], LAcc, HAcc) ->
+ split_locks(Locks, [{Name,{pkg,PkgName,Vsn},Lvl}|LAcc], HAcc);
+split_locks([{Name, {pkg,PkgName,Vsn,Hash}, Lvl} | Locks], LAcc, HAcc) ->
+ split_locks(Locks, [{Name,{pkg,PkgName,Vsn},Lvl}|LAcc], [{Name, Hash}|HAcc]);
+split_locks([Lock|Locks], LAcc, HAcc) ->
+ split_locks(Locks, [Lock|LAcc], HAcc).
consult_file(File) ->
Terms = consult_file_(File),